"""
weatherly
===============
A simple Python wrapper around WeatherAPI. Get current weather, forecast, history and more.

:copyright: (c) 2023-present @konradsic
:license: MIT license, see LICENSE for details
"""

__title__ = 'weatherly'
__author__ = 'konradsic'
__license__ = 'MIT'
__copyright__ = 'Copyright 2023-present konradsic'
__version__ = '0.9.0'

from typing import NamedTuple, Literal

from .api import *
from .errors import *
from .enums import *
from .responses import *
from . import (
    utils as utils
)

class VersionInfo(NamedTuple):
    major: int
    minor: int
    micro: int
    release_type: Literal['alpha', 'beta', 'candidate', 'final']
    
version_info = VersionInfo(major=0, minor=9, micro=0, release_type='final')

del NamedTuple, Literal, VersionInfo



"""
MIT License

Copyright (c) 2023 Konrad (@konradsic)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from enum import Enum

__all__ = (
    "Languages",
    "SportsEventType",
    "TideHeight",
)

class Languages(Enum):
    """Languages - an enum representing languages available for use in the WeatherAPI requests.
    
    Attributes are languages and language codes
    
    Attributes
    ----------
    Arabic: :class:`str`
        Language code: ar
    Bengali: :class:`str`
        Language code: bn
    Bulgarian: :class:`str`
        Language code: bg
    ChineseSimplified: :class:`str`
        Language code: zh
    ChineseTraditional: :class:`str`
        Language code: zh_tw
    Czech: :class:`str`
        Language code: cs
    Danish: :class:`str`
        Language code: da
    Dutch: :class:`str`
        Language code: nl
    Finnish: :class:`str`
        Language code: fi
    French: :class:`str`
        Language code: fr
    German: :class:`str`
        Language code: de
    Greek: :class:`str`
        Language code: el
    Hindi: :class:`str`
        Language code: hi
    Hungarian: :class:`str`
        Language code: hu
    Italian: :class:`str`
        Language code: it
    Japanese: :class:`str`
        Language code: ja
    Javanese: :class:`str`
        Language code: jv
    Korean: :class:`str`
        Language code: ko
    Mandarin: :class:`str`
        Language code: zh_cmn
    Marathi: :class:`str`
        Language code: mr
    Polish: :class:`str`
        Language code: pl
    Portuguese: :class:`str`
        Language code: pt
    Punjabi: :class:`str`
        Language code: pa
    Romanian: :class:`str`
        Language code: ro
    Russian: :class:`str`
        Language code: ru
    Serbian: :class:`str`
        Language code: sr
    Sinhalese: :class:`str`
        Language code: si
    Slovak: :class:`str`
        Language code: sk
    Spanish: :class:`str`
        Language code: es
    Swedish: :class:`str`
        Language code: sv
    Tamil: :class:`str`
        Language code: ta
    Telugu: :class:`str`
        Language code: te
    Turkish: :class:`str`
        Language code: tr
    Ukrainian: :class:`str`
        Language code: uk
    Urdu: :class:`str`
        Language code: ur
    Vietnamese: :class:`str`
        Language code: vi
    WuShanghainese: :class:`str`
        Language code: zh_wuu
    Xiang: :class:`str`
        Language code: zh_hsn
    YueCantonese: :class:`str`
        Language code: zh_yue
    Zulu: :class:`str`
        Language code: zu
    """
    Arabic: str = "ar"
    Bengali: str = "bn"
    Bulgarian: str = "bg"
    ChineseSimplified: str = "zh"
    ChineseTraditional: str = "zh_tw"
    Czech: str = "cs"
    Danish: str = "da"
    Dutch: str = "nl"
    Finnish: str = "fi"
    French: str = "fr"
    German: str = "de"
    Greek: str = "el"
    Hindi: str = "hi"
    Hungarian: str = "hu"
    Italian: str = "it"
    Japanese: str = "ja"
    Javanese: str = "jv"
    Korean: str = "ko"
    Mandarin: str = "zh_cmn"
    Marathi: str = "mr"
    Polish: str = "pl"
    Portuguese: str = "pt"
    Punjabi: str = "pa"
    Romanian: str = "ro"
    Russian: str = "ru"
    Serbian: str = "sr"
    Sinhalese: str = "si"
    Slovak: str = "sk"
    Spanish: str = "es"
    Swedish: str = "sv"
    Tamil: str = "ta"
    Telugu: str = "te"
    Turkish: str =	"tr"
    Ukrainian: str = "uk"
    Urdu: str = "ur"
    Vietnamese: str = "vi"
    WuShanghainese: str = "zh_wuu"
    Xiang: str = "zh_hsn"
    YueCantonese: str =	"zh_yue"
    Zulu: str = "zu"

class SportsEventType(Enum):
    """An enum representing type of sports event

    Attributes
    ---------------
    football
        Football event
    cricket
        Cricket event
    golf
        Golf event
    """
    football = 1
    cricket = 2
    golf = 3

    def __str__(self):
        return f"<SportsEventType: {self.name}>"

    def __repr__(self):
        return repr(self.__str__())

class TideHeight(Enum):
    """An enum representing tide height i.e. HIGH or LOW

    Attributes
    ----------------
    HIGH
        When the tide is high
    LOW
        When the tide is low
    """
    LOW = 0
    HIGH = 1


"""
MIT License

Copyright (c) 2023 Konrad (@konradsic)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

__all__ = (
    "WeatherlyException",
    "WeatherAPIException",
    "NoLocationFound",
    "InvalidAPIKey",
    "APILimitExceeded",
    "APIKeyDisabled",
    "AccessDenied",
    "InternalApplicationError",
    "InvalidDate",
)

class WeatherlyException(Exception):
    """
    A base class for all ``weatherly`` exceptions. Almost all exceptions of this module inherit from this class.

    Inherits from :class:`Exception`.
    """
    pass

class InvalidDate(WeatherlyException):
    """
    Raised when a date (for History/Future API) is invalid.

    Inherits from :class:`WeatherlyException`.
    """
    pass

class WeatherAPIException(WeatherlyException): 
    """
    The base class for :class:`Client` weather requests exceptions.
    
    Parameters
    ----------
    status: :class:`int`
        HTTP status code
    code: :class:`int`
        Error code
    message: :class:`str`
        Message provided with the error
    """
    def __init__(self, status: int, code: int, message: str, *args):
        self.status = status
        self.code = code
        self.message = message
        self.formatted_message = f"{self.status} (error code {self.code}): {message}"
        super().__init__(self.formatted_message, *args)

class NoLocationFound(WeatherAPIException): 
    """
    Exception like this is raised when no location was found when searching for weather data.

    Usually has status code  400 and error code 1006.
    
    Inherits from :class:`WeatherAPIException`.
    """
    pass

class InvalidAPIKey(WeatherAPIException): 
    """
    Exception like this is raised when provided API key is invalid

    Usually has status code 401 and error code 2006.
    
    Inherits from :class:`WeatherAPIException`.
    """
    pass

class APILimitExceeded(WeatherAPIException): 
    """
    Exception like this is raised when API key has exceeded monthly calls limit.

    Usually has status code 400 and error code 2007.
    
    Inherits from :class:`WeatherAPIException`.
    """
    pass

class APIKeyDisabled(WeatherAPIException): 
    """
    Exception like this is raised when API key has been disabled.

    Usually has status code 403 and error code 2008.
    
    Inherits from :class:`WeatherAPIException`.
    """
    pass

class AccessDenied(WeatherAPIException): 
    """
    Exception like this is raised when API key does not have access to requested resource.

    Usually has status code 403 and error code 2009.
    
    Inherits from :class:`WeatherAPIException`.
    """
    pass

class InternalApplicationError(WeatherAPIException): 
    """
    Exception like this is raised when an internal application error occured. There is nothing to do about it.

    Usually has status code 400 and error code 9999.
    
    Inherits from :class:`WeatherAPIException`.
    """
    pass


from setuptools import setup
import re

version = None
with open('weatherly/__init__.py') as f:
    version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', f.read(), re.MULTILINE).group(1)

if not version:
    raise RuntimeError('version is not set')

readme = ''
with open('README.rst', encoding='utf-8') as readme_file: 
    readme = readme_file.read()


requirements = []
with open('requirements.txt') as requirements_file: 
    requirements = requirements_file.read().splitlines()

extras_require = {
    'docs': [
        'sphinx==5.3.0',
        'sphinx_book_theme==1.0.1',
        'sphinxcontrib_trio==1.1.2'
    ],
    'test': [
        'pytest',
        'pytest-cov'
    ]
}

packages = [
    'weatherly',
    'weatherly.api',
]

setup(
    name='weatherly',
    author='konradsic',
    url='https://github.com/konradsic/weatherly',
    project_urls={
        'Issues': 'https://github.com/konradsic/weatherly/issues',
    },
    version=version,
    packages=packages,
    license='MIT',
    description='A simple Python wrapper around WeatherAPI. Get current weather, forecast, history and more...',
    long_description=readme,
    long_description_content_type='text/x-rst',
    include_package_data=True,
    install_requires=requirements,
    extras_require=extras_require,
    python_requires='>=3.10.0',
    classifiers=[
        'Development Status :: 1 - Planning',
        'License :: OSI Approved :: MIT License',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Topic :: Internet',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities',
        'Topic :: Scientific/Engineering :: Atmospheric Science',
        'Typing :: Typed',
    ],
)


"""
MIT License

Copyright (c) 2023 Konrad (@konradsic)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from typing import (
    Any,
    Optional,
    List,
    Tuple,
    Literal,
    Dict,
    TypeVar
)
import urllib
import requests
from ..utils import parse_kwargs_to_urlargs
import json

T = TypeVar("T")

__all__ = (
    "BaseAPIClient",
)

class BaseAPIClient():
    """
    Represents a base API client that can handle requests. Used as a base class for :class:`Client`
    
    Parameters
    ----------
    base_url: :class:`str`
        An URL that will be used as a base for requests.
    default_options: Optional[Dict[:class:`str`, Any]]
        Default options for requests made by the client that will be automatically added.
    """
    def __init__(
        self,
        base_url: str,
        default_options: Optional[Dict[str, Any]]
    ) -> None:
        self.url = base_url
        self.default_options = default_options or {}
        
    def _request(
        self,
        path: str,
        **kwargs: Optional[Dict[str, str]]
    ) -> Tuple[Dict[Any, Any], requests.Response]:
        """
        Request data from the base URL + path.
        Private function, use :class:`Client` methods instead
        
        Parameters
        ----------
        path: :class:`str`
            A path that will be used as an endpoint for the request.
        kwargs: Optional[Dict[str, str]]
            Additional parameters for the request.
        """
        full_url = self.url + path + parse_kwargs_to_urlargs({**self.default_options, **kwargs})

        response = requests.get(full_url)
        return response.json(), response
        
        
    